/*------------------------------------------------------------------------------------
   Name: xTDI Cross-EA.mq4
   Copyright 2010, Xaphod, http://www.xaphod.com
   
   Description: TDI green/red cross EA for backtesting. 
                For TDI cross entry with fixed SL & TP set: 
                    Order_Mode=0, SL=30, TP=40.
                For TDI cross entry and exit with protective SL & TP set:
                    Order_Mode=1, SL=30, TP=40.
                For TDI cross entry and exit only set: 
                    Order_Mode=1, SL=0, TP=0.
                NOTE! Requires that TDI-RT-Clone.mq4 is installed
	          
   Change log: 
       2014-03-23. Xaphod, v1.600
          - Upgraded to run in build 600+
       2012-03-22. Xaphod, v1.00 
          - First Release 
-------------------------------------------------------------------------------------*/
#property copyright "Copyright  2012, Xaphod"
#property link      "http://www.xaphod.com"

//#property strict
#property version    "1.600"

#property description "TDI green/red cross EA for backtesting purposes"
#property description " "
#property description "For TDI cross entry with fixed SL & TP set: Order_Mode=UseTPValue, SL=30, TP=40."
#property description "For TDI cross entry and exit with protective SL & TP set:Order_Mode=UseTDICross, SL=30, TP=40."
#property description "For TDI cross entry and exit only set: :Order_Mode=UseTDICross, SL=0, TP=0."
#property description " "
#property description "Requirements: The following indicators must installed and working: TDI-RT-Clone.mq4"
#property description "Troubleshooting: Check the 'Journal' and 'Expert' tabs in the terminal window for errors."


// Definitions
#define EA_NAME "xTDI Cross-EA"

#define TDI_INDI "TDI-RT-Clone"
#define STAT_IDLE 0     // Idle. waiting for setup bar
#define STAT_SETUP 1    // Setup bar active
#define STAT_STANDBY 2  // Waiting for entry. Setup bar closed.
#define STAT_TRADING 3  // Trade in progress

#define SIG_NO 0
#define SIG_UP 1
#define SIG_DN 2

#define TPMODE_SLTP   0
#define TPMODE_CROSS  1

enum ENUM_TP_MODE {
   UseTPValue=0,
   UseTDICross=1
};

//---- input parameters
extern string             RunTime_Settings=""; /* RunTime Settings */
extern string             RunTime_Enable="00:00";     /* Enable Trading */  // Enable trading after this time
extern string             RunTime_Disable="23:00";    /* Disable Trading */   // Disable trading after this time
extern string             Order_Settings="";/* Order Settings */
extern ENUM_TP_MODE       Order_TPMode=UseTDICross;                 /* Order_TPMode */// Take profit mode. Mode=0 Use TP value. Mode=1 Use TDI Cross to exit
extern double             Order_LotSize=0.1;              /* Order LotSize */// Lotsize
extern int                Order_SL=0;                     /* Order StopLoss (0=No SL) */ // Stoploss. Set to 0 to use no SL
extern int                Order_TP=0;                     /* Order TakeProfit (0=No TP) */ // Take profit. Set to 0 to use no TP
extern int                Order_MagicNr=2001;             /* Order Magic Number */
extern string             TDI_Settings="";/* TDI Settings */
extern int                TDI_RSIPeriod = 13;             /* TDI: RSI Period */
extern ENUM_APPLIED_PRICE TDI_RSIPrice = PRICE_CLOSE;     /* TDI: RSI Price Type */      
extern int                TDI_VolatilityBand = 34;        /* TDI: Volatility Band */
extern int                TDI_RSIPriceLine = 2;           /* TDI: RSI Price Line */  
extern ENUM_MA_METHOD     TDI_RSIPriceType = 0;           /* TDI: RSI Price Type */   
extern int                TDI_TradeSignalLine = 7;        /* TDI: TradeSignalLine */ 
extern ENUM_MA_METHOD     TDI_TradeSignalType = MODE_SMA; /* TDI: TradeSignalType */  

double gdPipValue;
int giPipDigits;
int giStatus;
int giTicket;
int giOrderType;
double gdSL;
double gdTP;


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init() {
  giStatus=STAT_IDLE;
  SetPipDigits();
  ResetOrderVars();
  RunTime_Enable=ValidateTime(RunTime_Enable);
  RunTime_Disable=ValidateTime(RunTime_Disable);
  return(0);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit() {
   return(0);
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()  {
  datetime tEnableTime;
  datetime tDisableTime;
  
  // Check that the pip value has been set
  if (gdPipValue==0) {
    SetPipDigits();
    return(0);
  }
  
  // Check Session time
  tEnableTime=StrToTime(TimeToStr(Time[0],TIME_DATE )+" "+RunTime_Enable);
  tDisableTime=StrToTime(TimeToStr(Time[0],TIME_DATE )+" "+RunTime_Disable);
  if (giStatus==STAT_IDLE && (Time[0]<tEnableTime || Time[0]>tDisableTime))
    return(0);
  
  // State
  // Manage open trade
  if ( giStatus==STAT_TRADING) {
      CheckSL();
      CheckTP();
  }
  // Check for signal if idle
  if (giStatus==STAT_IDLE) {
    if (TDICrossSig(1)==SIG_UP)
      OpenPosition(OP_BUY);
    if (TDICrossSig(1)==SIG_DN)
      OpenPosition(OP_SELL);
  }
   
  return(0);
}
//+------------------------------------------------------------------+

//-----------------------------------------------------------------------------
// function: TDIStrong()
// Description: TDI green over/under red/yellow/50
//-----------------------------------------------------------------------------
int TDICrossSig(int i) {
  double dRSIPriceLine0,dRSIPriceLine1;
  double dTradeSignalLine0,dTradeSignalLine1;
  
  dRSIPriceLine0=iCustom(NULL,0,TDI_INDI,TDI_RSIPeriod,TDI_RSIPrice,TDI_VolatilityBand,TDI_RSIPriceLine,
                        TDI_RSIPriceType,TDI_TradeSignalLine,TDI_TradeSignalType,4,i);     // Green line
  dRSIPriceLine1=iCustom(NULL,0,TDI_INDI,TDI_RSIPeriod,TDI_RSIPrice,TDI_VolatilityBand,TDI_RSIPriceLine,
                        TDI_RSIPriceType,TDI_TradeSignalLine,TDI_TradeSignalType,4,i+1); 
  dTradeSignalLine0=iCustom(NULL,0,TDI_INDI,TDI_RSIPeriod,TDI_RSIPrice,TDI_VolatilityBand,TDI_RSIPriceLine,
                           TDI_RSIPriceType,TDI_TradeSignalLine,TDI_TradeSignalType,3,i);  // Red line
  dTradeSignalLine1=iCustom(NULL,0,TDI_INDI,TDI_RSIPeriod,TDI_RSIPrice,TDI_VolatilityBand,TDI_RSIPriceLine,
                           TDI_RSIPriceType,TDI_TradeSignalLine,TDI_TradeSignalType,3,i+1); 
        
  if (dRSIPriceLine0>dTradeSignalLine0 && dRSIPriceLine1<dTradeSignalLine1)
    return(SIG_UP);
  else if (dRSIPriceLine0<dTradeSignalLine0 && dRSIPriceLine1>dTradeSignalLine1)
    return(SIG_DN);
  else
    return(SIG_NO);
}


//-----------------------------------------------------------------------------
// function: ResetOrderVars()
// Description: Reset Order variables
//-----------------------------------------------------------------------------
void ResetOrderVars() {
  giOrderType=-1;
  giTicket=-1;
  gdSL=0;
  gdTP=0;  
  return;
}


//-----------------------------------------------------------------------------
// function: CheckSL()
// Description: Check if SL has been hit
//-----------------------------------------------------------------------------
void CheckSL() {
  // Close if SL is hit
  if (Order_SL>0) {
    if (giOrderType==OP_BUY && Bid<=gdSL)
      ClosePosition(Bid);
    if (giOrderType==OP_SELL && Ask>=gdSL)
      ClosePosition(Ask);
  }
  return;
}


//-----------------------------------------------------------------------------
// function: CheckTP()
// Description: Check if TP has been hit
//-----------------------------------------------------------------------------
void CheckTP() {
  
  // Close if TP is hit
  if (Order_TP>0) {
    if (giOrderType==OP_BUY && Bid>=gdTP)
      ClosePosition(Bid);
    if (giOrderType==OP_SELL && Ask<=gdTP)
      ClosePosition(Ask);
  }
  
  // Mode 1. Close when TDI status changes
  if (Order_TPMode==TPMODE_CROSS) {     
    if (giOrderType==OP_BUY && TDICrossSig(1)==SIG_DN)
      ClosePosition(Bid);
    if (giOrderType==OP_SELL && TDICrossSig(1)==SIG_UP)
      ClosePosition(Ask);
  }
  return;
}


//-----------------------------------------------------------------------------
// function: OpenPosition()
// Description: Convert a price difference to pips.
//-----------------------------------------------------------------------------
int OpenPosition(int iOrderType) {
  int iError;
  
  // Check account balance is sufficent
  if(AccountFreeMargin()<(1000*Order_LotSize))  {
    Print("TDI-EA: No money available. Free Margin = ", AccountFreeMargin());
    return(0);      
  }    
  WaitForTradeContext();
  RefreshRates();
  
  // Check for long position (buy) possibility
  if(iOrderType==OP_BUY && giStatus==STAT_IDLE ) {
        
    // Open order    
    gdSL=NormalizeDouble(Ask-Order_SL*gdPipValue,Digits);
    gdTP=NormalizeDouble(Ask+Order_TP*gdPipValue,Digits);
      
    giTicket=OrderSend(Symbol(),OP_BUY,Order_LotSize,Ask,3,0,0,"",Order_MagicNr,0,Green);
    if(giTicket>=0) {
      if(OrderSelect(giTicket,SELECT_BY_TICKET,MODE_TRADES)) { 
        Print(EA_NAME+" BUY order opened: ",OrderOpenPrice()," ",Symbol());
        giStatus=STAT_TRADING;
        giOrderType=iOrderType;
      }
      else {
        iError=GetLastError();
        Print(EA_NAME+" Error opening BUY order: ",iError," ",Symbol());
        Print(EA_NAME+" Error Buy Open:",Ask, "  SL: ",gdSL,"  TP: ", gdTP);
        giStatus=STAT_IDLE;
        ResetOrderVars();
        return(0); 
      }
    }
    else {   
      iError=GetLastError();
      Print(EA_NAME+" Error opening BUY order: ",iError," ",Symbol());
      Print(EA_NAME+" Error Buy Open:",Ask, "  SL: ",gdSL,"  TP: ", gdTP);
      giStatus=STAT_IDLE;
      ResetOrderVars();
      return(0); 
    }
  }
  // Check for short position (sell) possibility
  else if(iOrderType==OP_SELL && giStatus==STAT_IDLE ) {    
    // Open order    
    gdSL=NormalizeDouble(Bid+Order_SL*gdPipValue,Digits);
    gdTP=NormalizeDouble(Bid-Order_TP*gdPipValue,Digits);
      
    giTicket=OrderSend(Symbol(),OP_SELL,Order_LotSize,Bid,3,0,0,"",Order_MagicNr,0,Green);
    if(giTicket>=0) {
      if(OrderSelect(giTicket,SELECT_BY_TICKET,MODE_TRADES)) { 
        Print(EA_NAME+" SELL order opened: ",OrderOpenPrice()," ",Symbol());
        giStatus=STAT_TRADING;
        giOrderType=iOrderType;
      }
      else {
        iError=GetLastError();
        Print(EA_NAME+" Error opening SELL order: ",iError," ",Symbol());
        Print(EA_NAME+" Error Sell Open:",Bid, "  SL: ",gdSL,"  TP: ", gdTP);
        giStatus=STAT_IDLE;
        ResetOrderVars();
        return(0); 
      }
    }
    else {   
      iError=GetLastError();
      Print(EA_NAME+" Error opening SELL order: ",iError," ",Symbol());
      Print(EA_NAME+" Error Sell Open:",Bid, "  SL: ",gdSL,"  TP: ", gdTP);
      giStatus=STAT_IDLE;
      ResetOrderVars();
      return(0); 
    }
  }
  return(0); 
}


//-----------------------------------------------------------------------------
// function: ClosePosition()
// Description: Close an open positio
//-----------------------------------------------------------------------------
void ClosePosition(double dPrice) {
  int iError;
  
  WaitForTradeContext();
  RefreshRates();
  if (OrderClose(giTicket,Order_LotSize,dPrice,3,Red)) {
    Print(EA_NAME+" Order Closed:",dPrice);
    giStatus=STAT_IDLE;
    ResetOrderVars();
  }
  else {
    iError=GetLastError();
    Print(EA_NAME+" Error closing position. ErrNr=",iError,", Ticket=",giTicket,", Price",DoubleToStr(dPrice,Digits),", ",Symbol());
    
  }
  return;
}


//-----------------------------------------------------------------------------
// function: WaitForTradeContext()
// Description: Wait for TradeContext to be released
//-----------------------------------------------------------------------------
int WaitForTradeContext(int iTimeOut=120) {
  datetime dStartTime;
  
  // Don't bother with the TradeContextBusy if in test mode 
  if(IsTesting()) 
    return(1);
     
  if(!IsTradeContextBusy()) 
    return(0);
  else
    Comment("Trade Context Busy. Waiting....");
    
  // Wait for tradeflag to be released or timeout.
  dStartTime=GetTickCount(); // Reset timeout timer
  while(true) {
    if(!IsTradeContextBusy()) 
      return(0);
    // Exit if EA is stopped
    if (IsStopped()==true) 
      return(-1);    
    // Exit on time out
    if ((GetTickCount()-dStartTime)>(iTimeOut*1000) )
      return(-3);    
    Sleep(50);    
  } // endwhile
  return(-3);
}


//-----------------------------------------------------------------------------
// function: SetPipDigits()
// Description: Set the number of digits for the pip value
//-----------------------------------------------------------------------------
void SetPipDigits() {
  int iaPipMult[]={1,10,1,10,1,10,100};
  gdPipValue = Point * iaPipMult[Digits];
  giPipDigits=MathLog(iaPipMult[Digits])/MathLog(10);
}


//-----------------------------------------------------------------------------
// function: ValidateTime()
// Description: Return a valid time format (ex. 08:30) or an error.
//-----------------------------------------------------------------------------
string ValidateTime(string sTime) {  
  if (StringLen(sTime)==1) return("0"+sTime+":00");
  if (StringLen(sTime)==2) return(sTime+":00");
  if (StringLen(sTime)==4 && StringSubstr(sTime, 1, 1)==":")  return("0"+sTime);
  if (StringLen(sTime)==4) return(StringSubstr(sTime, 0, 2)+":"+StringSubstr(sTime, 2, 2));
  if (StringLen(sTime)==5 && StringSubstr(sTime, 2, 1)==":") return(sTime);
  if (StringLen(sTime)==5 && StringSubstr(sTime, 2, 1)==".") return(StringSubstr(sTime, 0, 2)+":"+StringSubstr(sTime, 3, 2));
  // Error warning 
  Alert(Symbol()," Error: Invalid Time Format in input parameters! ",sTime); 
  return(sTime);
}


//-----------------------------------------------------------------------------
// function: TF2Str()
// Description: Convert time-frame to a string
//-----------------------------------------------------------------------------
string TF2Str(int iPeriod) {
  switch(iPeriod) {
    case PERIOD_M1: return("M1");
    case PERIOD_M5: return("M5");
    case PERIOD_M15: return("M15");
    case PERIOD_M30: return("M30");
    case PERIOD_H1: return("H1");
    case PERIOD_H4: return("H4");
    case PERIOD_D1: return("D1");
    case PERIOD_W1: return("W1");
    case PERIOD_MN1: return("MN1");
    default: return("M"+iPeriod);
  }
}


